#!/bin/sh
# For Linux systems
# Script to detect high memory regions used by running Squeak processes
# Ned Konz Sun Jul 11 11:23:38 PDT 2004
pmap -x `pgrep squeak` | gawk --non-decimal-data '
BEGIN { problems2 = 0; problems4 = 0; OFS="\n"; twoGiB = 2 * 1024 * 1024 * 1024; fourGiB = 2 * twoGiB; }
/^[0-9]+:/ { header = $0 }
$6 ~ /rw-/ && $2 > 10000 {
	bytes = $2 * 1024
	startaddr = strtonum("0x" $1)
	endaddr = startaddr + bytes
	if (startaddr >= twoGiB || endaddr > twoGiB)
	{ problems2++; if (header) { print "PID " header; header = "" }
	printf("%08x-%08x %s\n", startaddr, endaddr, substr($0,9,1000)) }
	if (startaddr >= fourGiB || endaddr > fourGiB)
	{ problems4++; if (header) { print "PID " header; header = "" } 
	printf("%08x-%08x %s\n", startaddr, endaddr, substr($0,9,1000)) }
	}
END {
	if (problems2 + problems4 > 0) { print "",
		"possible problems:",
		problems2 " memory regions that start or extend past the 2GiB boundary were found.",
		problems4 " memory regions that start or extend past the 4GiB boundary were found.",
		"Squeak may not run properly on your system.",
		"Consider using the --mmap command line switch",
		"to limit the amount of virtual memory used by Squeak." }
	else { print "Memory seems to be OK" }
	exit problems2 + problems4 }'
